@@ -68,6 +68,13 @@
 #define XML_SGML_DEFAULT_CATALOG "file:///etc/sgml/catalog"
 #endif
 
+#if defined(_WIN32) && defined(_MSC_VER)
+#undef XML_XML_DEFAULT_CATALOG
+static char XML_XML_DEFAULT_CATALOG[256] = "file:///etc/xml/catalog";
+void* __stdcall GetModuleHandleA(const char*);
+unsigned long __stdcall GetModuleFileNameA(void*, char*, unsigned long);
+#endif
+
 static int xmlExpandCatalog(xmlCatalogPtr catal, const char *filename);
 
 /************************************************************************
@@ -2731,7 +2738,7 @@ xmlACatalogResolveURI(xmlCatalogPtr catal, const xmlChar *URI) {
  * @catal:  a Catalog
  * @out:  the file.
  *
- * Free up all the memory associated with catalogs
+ * Dump the given catalog to the given file.
  */
 void
 xmlACatalogDump(xmlCatalogPtr catal, FILE *out) {
@@ -2925,7 +2932,35 @@ xmlInitializeCatalog(void) {
 
 	catalogs = (const char *) getenv("XML_CATALOG_FILES");
 	if (catalogs == NULL)
+#if defined(_WIN32) && defined(_MSC_VER)
+    {
+		void* hmodule;
+		hmodule = GetModuleHandleA("libxml2.dll");
+		if (hmodule == NULL)
+			hmodule = GetModuleHandleA(NULL);
+		if (hmodule != NULL) {
+			char buf[256];
+			unsigned long len = GetModuleFileNameA(hmodule, buf, 255);
+			if (len != 0) {
+				char* p = &(buf[len]);
+				while (*p != '\\' && p > buf) 
+					p--;
+				if (p != buf) {
+					xmlChar* uri;
+					strncpy(p, "\\..\\etc\\catalog", 255 - (p - buf));
+					uri = xmlCanonicPath(buf);
+					if (uri != NULL) {
+						strncpy(XML_XML_DEFAULT_CATALOG, uri, 255);
+						xmlFree(uri);
+					}
+				}
+			}
+		}
+		catalogs = XML_XML_DEFAULT_CATALOG;
+    }
+#else
 	    catalogs = XML_XML_DEFAULT_CATALOG;
+#endif
 
 	catal = xmlCreateNewCatalog(XML_XML_CATALOG_TYPE, 
 		xmlCatalogDefaultPrefer);
@@ -3147,7 +3182,7 @@ xmlCatalogResolveURI(const xmlChar *URI) {
  * xmlCatalogDump:
  * @out:  the file.
  *
- * Free up all the memory associated with catalogs
+ * Dump all the global catalog content to the given file.
  */
 void
 xmlCatalogDump(FILE *out) {
