@@ -16,6 +16,8 @@
 #include <libxml/encoding.h>
 #include <libxml/xmlwriter.h>
 
+#if defined(LIBXML_WRITER_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)
+
 #define MY_ENCODING "ISO-8859-1"
 
 void testXmlwriterFilename(const char *uri);
@@ -78,7 +80,7 @@ testXmlwriterFilename(const char *uri)
     }
 
     /* Start the document with the xml default for the version,
-     * encoding ISO 8858-1 and the default for the standalone
+     * encoding ISO 8859-1 and the default for the standalone
      * declaration. */
     rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
     if (rc < 0) {
@@ -357,7 +359,7 @@ testXmlwriterMemory(const char *file)
     }
 
     /* Start the document with the xml default for the version,
-     * encoding ISO 8858-1 and the default for the standalone
+     * encoding ISO 8859-1 and the default for the standalone
      * declaration. */
     rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
     if (rc < 0) {
@@ -634,7 +636,7 @@ testXmlwriterDoc(const char *file)
     }
 
     /* Start the document with the xml default for the version,
-     * encoding ISO 8858-1 and the default for the standalone
+     * encoding ISO 8859-1 and the default for the standalone
      * declaration. */
     rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
     if (rc < 0) {
@@ -906,7 +908,7 @@ testXmlwriterTree(const char *file)
     }
 
     /* Start the document with the xml default for the version,
-     * encoding ISO 8858-1 and the default for the standalone
+     * encoding ISO 8859-1 and the default for the standalone
      * declaration. */
     rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
     if (rc < 0) {
@@ -1187,3 +1189,10 @@ ConvertInput(const char *in, const char *encoding)
 
     return out;
 }
+
+#else
+int main(void) {
+    fprintf(stderr, "Writer or output support not compiled in\n");
+    exit(1);
+}
+#endif
