@@ -4,6 +4,7 @@
 #
 
 functions = {}
+enums = {} # { enumType: { enumConstant: enumValue } }
 
 import sys
 import string
@@ -137,7 +138,8 @@ class docParser:
                     self.function_return_info = attrs['info']
                 if attrs.has_key('field'):
                     self.function_return_field = attrs['field']
-
+        elif tag == 'enum':
+            enum(attrs['type'],attrs['name'],attrs['value'])
 
     def end(self, tag):
         if debug:
@@ -167,10 +169,13 @@ class docParser:
                 
                 
 def function(name, desc, ret, args, file):
-    global functions
-
     functions[name] = (desc, ret, args, file)
 
+def enum(type, name, value):
+    if not enums.has_key(type):
+        enums[type] = {}
+    enums[type][name] = value
+
 #######################################################################
 #
 #  Some filtering rukes to drop functions/types which should not
@@ -268,6 +273,7 @@ py_types = {
     'xmlCatalogPtr': ('O', "catalog", "xmlCatalogPtr", "xmlCatalogPtr"),
     'FILE *': ('O', "File", "FILEPtr", "FILE *"),
     'xmlURIPtr': ('O', "URI", "xmlURIPtr", "xmlURIPtr"),
+    'xmlErrorPtr': ('O', "Error", "xmlErrorPtr", "xmlErrorPtr"),
     'xmlOutputBufferPtr': ('O', "outputBuffer", "xmlOutputBufferPtr", "xmlOutputBufferPtr"),
     'xmlParserInputBufferPtr': ('O', "inputBuffer", "xmlParserInputBufferPtr", "xmlParserInputBufferPtr"),
     'xmlRegexpPtr': ('O', "xmlReg", "xmlRegexpPtr", "xmlRegexpPtr"),
@@ -296,6 +302,8 @@ def skip_function(name):
         return 1
     if name == "xmlFreeParserCtxt":
         return 1
+    if name == "xmlCleanupParser":
+        return 1
     if name == "xmlFreeTextReader":
         return 1
 #    if name[0:11] == "xmlXPathNew":
@@ -321,6 +329,12 @@ def skip_function(name):
         return 1
     if name == "xmlTextReaderValue":
         return 1
+    if name == "xmlOutputBufferClose": # handled by by the superclass
+        return 1
+    if name == "xmlOutputBufferFlush": # handled by by the superclass
+        return 1
+    if name == "xmlErrMemory":
+        return 1
     return 0
 
 def print_function_wrapper(name, output, export, include):
@@ -420,7 +434,7 @@ def print_function_wrapper(name, output, export, include):
         include.write("#ifdef LIBXML_DEBUG_ENABLED\n");
         export.write("#ifdef LIBXML_DEBUG_ENABLED\n");
         output.write("#ifdef LIBXML_DEBUG_ENABLED\n");
-    elif file == "HTMLtree" or file == "HTMLparser":
+    elif file == "HTMLtree" or file == "HTMLparser" or name[0:4] == "html":
         include.write("#ifdef LIBXML_HTML_ENABLED\n");
         export.write("#ifdef LIBXML_HTML_ENABLED\n");
         output.write("#ifdef LIBXML_HTML_ENABLED\n");
@@ -458,16 +472,25 @@ def print_function_wrapper(name, output, export, include):
 
     if file == "python":
         # Those have been manually generated
+        if name[0:4] == "html":
+	    include.write("#endif /* LIBXML_HTML_ENABLED */\n");
+	    export.write("#endif /* LIBXML_HTML_ENABLED */\n");
+	    output.write("#endif /* LIBXML_HTML_ENABLED */\n");
         return 1
     if file == "python_accessor" and ret[0] != "void" and ret[2] is None:
         # Those have been manually generated
+        if name[0:4] == "html":
+	    include.write("#endif /* LIBXML_HTML_ENABLED */\n");
+	    export.write("#endif /* LIBXML_HTML_ENABLED */\n");
+	    output.write("#endif /* LIBXML_HTML_ENABLED */\n");
         return 1
 
     output.write("PyObject *\n")
-    output.write("libxml_%s(ATTRIBUTE_UNUSED PyObject *self," % (name))
+    output.write("libxml_%s(PyObject *self ATTRIBUTE_UNUSED," % (name))
+    output.write(" PyObject *args")
     if format == "":
-	output.write("ATTRIBUTE_UNUSED ")
-    output.write(" PyObject *args) {\n")
+	output.write(" ATTRIBUTE_UNUSED")
+    output.write(") {\n")
     if ret[0] != 'void':
         output.write("    PyObject *py_retval;\n")
     if c_return != "":
@@ -488,7 +511,7 @@ def print_function_wrapper(name, output, export, include):
         include.write("#endif /* LIBXML_DEBUG_ENABLED */\n");
         export.write("#endif /* LIBXML_DEBUG_ENABLED */\n");
         output.write("#endif /* LIBXML_DEBUG_ENABLED */\n");
-    elif file == "HTMLtree" or file == "HTMLparser":
+    elif file == "HTMLtree" or file == "HTMLparser" or name[0:4] == "html":
         include.write("#endif /* LIBXML_HTML_ENABLED */\n");
         export.write("#endif /* LIBXML_HTML_ENABLED */\n");
         output.write("#endif /* LIBXML_HTML_ENABLED */\n");
@@ -568,7 +591,6 @@ def buildStubs():
     wrapper = open("libxml2-py.c", "w")
     wrapper.write("/* Generated */\n\n")
     wrapper.write("#include <Python.h>\n")
-#    wrapper.write("#include \"config.h\"\n")
     wrapper.write("#include <libxml/xmlversion.h>\n")
     wrapper.write("#include <libxml/tree.h>\n")
     wrapper.write("#include <libxml/xmlschemastypes.h>\n")
@@ -635,6 +657,7 @@ classes_type = {
     "htmlParserCtxt *": ("._o", "parserCtxt(_obj=%s)", "parserCtxt"),
     "xmlCatalogPtr": ("._o", "catalog(_obj=%s)", "catalog"),
     "xmlURIPtr": ("._o", "URI(_obj=%s)", "URI"),
+    "xmlErrorPtr": ("._o", "Error(_obj=%s)", "Error"),
     "xmlOutputBufferPtr": ("._o", "outputBuffer(_obj=%s)", "outputBuffer"),
     "xmlParserInputBufferPtr": ("._o", "inputBuffer(_obj=%s)", "inputBuffer"),
     "xmlRegexpPtr": ("._o", "xmlReg(_obj=%s)", "xmlReg"),
@@ -715,6 +738,9 @@ def nameFixup(name, classe, type, file):
     elif name[0:9] == "xmlURISet" and file == "python_accessor":
         func = name[6:]
         func = string.lower(func[0:1]) + func[1:]
+    elif name[0:11] == "xmlErrorGet" and file == "python_accessor":
+        func = name[11:]
+        func = string.lower(func[0:1]) + func[1:]
     elif name[0:17] == "xmlXPathParserGet" and file == "python_accessor":
         func = name[17:]
         func = string.lower(func[0:1]) + func[1:]
@@ -994,7 +1020,7 @@ def buildWrappers():
 		    rlist = reference_keepers[classname]
 		    for ref in rlist:
 		        classes.write("        self.%s = None\n" % ref[1])
-		classes.write("        self._o = None\n")
+		classes.write("        self._o = _obj\n")
 		classes.write("        %s.__init__(self, _obj=_obj)\n\n" % (
 			      classes_ancestor[classname]))
 		if classes_ancestor[classname] == "xmlCore" or \
@@ -1013,17 +1039,25 @@ def buildWrappers():
 		        classes.write("        self.%s = None\n" % ref[1])
 		classes.write("        if _obj != None:self._o = _obj;return\n")
 		classes.write("        self._o = None\n\n");
+	    destruct=None
 	    if classes_destructors.has_key(classname):
 		classes.write("    def __del__(self):\n")
 		classes.write("        if self._o != None:\n")
 		classes.write("            libxml2mod.%s(self._o)\n" %
 			      classes_destructors[classname]);
 		classes.write("        self._o = None\n\n");
+		destruct=classes_destructors[classname]
 	    flist = function_classes[classname]
 	    flist.sort(functionCompare)
 	    oldfile = ""
 	    for info in flist:
 		(index, func, name, ret, args, file) = info
+		#
+		# Do not provide as method the destructors for the class
+		# to avoid double free
+		#
+		if name == destruct:
+		    continue;
 		if file != oldfile:
 		    if file == "python_accessor":
 			classes.write("    # accessors for %s\n" % (classname))
@@ -1150,9 +1184,19 @@ def buildWrappers():
 			classes.write("        return ret\n");
 		classes.write("\n");
 
+    #
+    # Generate enum constants
+    #
+    for type,enum in enums.items():
+        classes.write("# %s\n" % type)
+        items = enum.items()
+        items.sort(lambda i1,i2: cmp(long(i1[1]),long(i2[1])))
+        for name,value in items:
+            classes.write("%s = %s\n" % (name,value))
+        classes.write("\n");
+
     txt.close()
     classes.close()
 
-
 buildStubs()
 buildWrappers()
