@@ -365,36 +365,16 @@ function genReadme(bname, ver, file)
 	f.WriteLine("  This is " + bname + ", version " + ver + ", binary package for the native Win32/IA32");
 	f.WriteLine("platform.");
 	f.WriteBlankLines(1);
-	f.WriteLine("  The directory named 'include' contains the header files. Place its");
-	f.WriteLine("contents somewhere where it can be found by the compiler.");
-	f.WriteLine("  The directory which answers to the name 'lib' contains the static and");
-	f.WriteLine("dynamic libraries. Place them somewhere where they can be found by the");
-	f.WriteLine("linker. The files whose names end with '_a.lib' are aimed for static");
-	f.WriteLine("linking, the other files are lib/dll pairs.");
-	f.WriteLine("  The directory called 'util' contains various programs which count as a");
-	f.WriteLine("part of " + bname + ".");
+	f.WriteLine("  The files in this package do not require any special installation");
+	f.WriteLine("steps. Extract the contents of the archive whereever you wish and");
+	f.WriteLine("make sure that your tools which use " + bname + " can find it.");
 	f.WriteBlankLines(1);
-	f.WriteLine("  If you plan to develop your own programme, in C, which uses " + bname + ", then");
-	f.WriteLine("you should know what to do with the files in the binary package. If you don't,");
-	f.WriteLine("know this, then please, please do some research on how to use a");
-	f.WriteLine("third-party library in a C programme. The topic belongs to the very basics"); 
-	f.WriteLine("and you will not be able to do much without that knowledge.");
-	f.WriteBlankLines(1);
-	f.WriteLine("  If you wish to use " + bname + " solely through the supplied utilities, such as");
-	f.WriteLine("xmllint or xsltproc, then all you need to do is place the");
-	f.WriteLine("contents of the 'lib' and 'util' directories from the binary package in a"); 
-	f.WriteLine("directory on your disc which is mentioned in your PATH environment"); 
-	f.WriteLine("variable. You can use an existing directory which is allready in the"); 
-	f.WriteLine("path, such as 'C:\WINDOWS', or 'C:\WINNT'. You can also create a new"); 
-	f.WriteLine("directory for " + bname + " and place the files there, but be sure to modify"); 
-	f.WriteLine("the PATH environment variable and add that new directory to its list.");
-	f.WriteBlankLines(1);
-	f.WriteLine("  If you use other software which needs " + bname + ", such as Apache");
-	f.WriteLine("Web Server in certain configurations, then please consult the"); 
-	f.WriteLine("documentation of that software and see if it mentions something about");
-	f.WriteLine("how it uses " + bname + " and how it expects it to be installed. If you find");
-	f.WriteLine("nothing, then the default installation, as described in the previous"); 
-	f.WriteLine("paragraph, should be suficient.");
+	f.WriteLine("  For example, if you want to run the supplied utilities from the command");
+	f.WriteLine("line, you can, if you wish, add the 'bin' subdirectory to the PATH");
+	f.WriteLine("environment variable.");
+	f.WriteLine("  If you want to make programmes in C which use " + bname + ", you'll");
+	f.WriteLine("likely know how to use the contents of this package. If you don't, please");
+	f.WriteLine("refer to your compiler's documentation."); 
 	f.WriteBlankLines(1);
 	f.WriteLine("  If there is something you cannot keep for yourself, such as a problem,");
 	f.WriteLine("a cheer of joy, a comment or a suggestion, feel free to contact me using");
