@@ -81,6 +81,7 @@ struct _xmlXIncludeCtxt {
     int              nbErrors; /* the number of errors detected */
     int                legacy; /* using XINCLUDE_OLD_NS */
     int            parseFlags; /* the flags used for parsing XML documents */
+    xmlChar *		 base; /* the current xml:base */
 };
 
 static int
@@ -677,6 +678,11 @@ xmlXIncludeRecurseDoc(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc,
 	newctxt->urlNr = ctxt->urlNr;
 	newctxt->urlTab = ctxt->urlTab;
 
+	/*
+	 * Inherit the existing base
+	 */
+	newctxt->base = ctxt->base;
+
 	/*
 	 * Inherit the documents already in use by other includes
 	 */
@@ -1557,6 +1563,7 @@ loaded:
 		    xmlFree(fragment);
 		    return(-1);
 		}
+
 	    case XPATH_RANGE:
 	    case XPATH_LOCATIONSET:
 		break;
@@ -1569,7 +1576,6 @@ loaded:
 		switch (set->nodeTab[i]->type) {
 		    case XML_TEXT_NODE:
 		    case XML_CDATA_SECTION_NODE:
-		    case XML_ELEMENT_NODE:
 		    case XML_ENTITY_REF_NODE:
 		    case XML_ENTITY_NODE:
 		    case XML_PI_NODE:
@@ -1580,6 +1586,19 @@ loaded:
 		    case XML_DOCB_DOCUMENT_NODE:
 #endif
 			continue;
+		    case XML_ELEMENT_NODE: {
+			xmlChar *nodeBase;
+			xmlNodePtr el = set->nodeTab[i];
+
+			nodeBase = xmlNodeGetBase(el->doc, el);
+			if (nodeBase != NULL) {
+			    if (!xmlStrEqual(nodeBase, el->doc->URL))
+			        xmlNodeSetBase(el, nodeBase);
+			    xmlFree(nodeBase);
+			}
+			continue;
+		    }
+
 		    case XML_ATTRIBUTE_NODE:
 			xmlXIncludeErr(ctxt, ctxt->incTab[nr]->ref, 
 			               XML_XINCLUDE_XPTR_RESULT,
@@ -1631,12 +1650,32 @@ loaded:
      */
     if ((doc != NULL) && (URL != NULL) && (xmlStrchr(URL, (xmlChar) '/'))) {
 	xmlNodePtr node;
+	xmlChar *relURI;
+	xmlChar *curBase;
 
-	node = ctxt->incTab[nr]->inc;
-	while (node != NULL) {
-	    if (node->type == XML_ELEMENT_NODE)
-		xmlNodeSetBase(node, URL);
-	    node = node->next;
+	/*
+	 * The base is only adjusted if necessary for the existing base
+	 */
+	relURI = xmlBuildRelativeURI(URL, ctxt->base);
+	if (relURI == NULL) {	/* Error return */
+	    xmlXIncludeErr(ctxt, ctxt->incTab[nr]->ref, 
+	               XML_XINCLUDE_HREF_URI,
+		       "trying to build relative URI from %s\n", URL);
+	} else {
+	    if (xmlStrchr(relURI, (xmlChar) '/')) {
+		node = ctxt->incTab[nr]->inc;
+		while (node != NULL) {
+	            if (node->type == XML_ELEMENT_NODE) {
+			curBase = xmlNodeGetBase(node->doc, node);
+			if ((curBase == NULL) || xmlStrEqual(curBase, node->doc->URL))
+			    xmlNodeSetBase(node, relURI);
+			if (curBase != NULL)
+			    xmlFree(curBase);
+		    }
+	            node = node->next;
+		}
+	    }
+	    xmlFree(relURI);
 	}
     }
     if ((nr < ctxt->incNr) && (ctxt->incTab[nr]->doc != NULL) &&
@@ -1814,6 +1853,7 @@ xmlXIncludeLoadFallback(xmlXIncludeCtxtPtr ctxt, xmlNodePtr fallback, int nr) {
 	newctxt = xmlXIncludeNewContext(ctxt->doc);
 	if (newctxt == NULL)
 	    return (-1);
+	newctxt->base = ctxt->base;	/* Inherit the base from the existing context */
 	xmlXIncludeSetFlags(newctxt, ctxt->parseFlags);
 	ret = xmlXIncludeDoProcess(newctxt, ctxt->doc, fallback->children);
 	if (ctxt->nbErrors > 0)
@@ -1867,6 +1907,7 @@ xmlXIncludeLoadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
     xmlChar *href;
     xmlChar *parse;
     xmlChar *base;
+    xmlChar *oldBase;
     xmlChar *URI;
     int xml = 1; /* default Issue 64 */
     int ret;
@@ -1947,14 +1988,23 @@ xmlXIncludeLoadNode(xmlXIncludeCtxtPtr ctxt, int nr) {
 #endif
 
     /*
-     * Cleanup
+     * Save the base for this include (saving the current one)
      */
+    oldBase = ctxt->base;
+    ctxt->base = base;
+
     if (xml) {
 	ret = xmlXIncludeLoadDoc(ctxt, URI, nr);
 	/* xmlXIncludeGetFragment(ctxt, cur, URI); */
     } else {
 	ret = xmlXIncludeLoadTxt(ctxt, URI, nr);
     }
+
+    /*
+     * Restore the original base before checking for fallback
+     */
+    ctxt->base = oldBase;
+    
     if (ret < 0) {
 	xmlNodePtr children;
 
@@ -2194,7 +2244,7 @@ xmlXIncludeDoProcess(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc, xmlNodePtr tree) {
     cur = tree;
     if (xmlXIncludeTestNode(ctxt, cur) == 1)
 	xmlXIncludePreProcessNode(ctxt, cur);
-    while (cur != NULL) {
+    while ((cur != NULL) && (cur != tree->parent)) {
 	/* TODO: need to work on entities -> stack */
 	if ((cur->children != NULL) &&
 	    (cur->children->type != XML_ENTITY_DECL) &&
@@ -2212,7 +2262,8 @@ xmlXIncludeDoProcess(xmlXIncludeCtxtPtr ctxt, xmlDocPtr doc, xmlNodePtr tree) {
 	        break;
 	    do {
 		cur = cur->parent;
-		if (cur == NULL) break; /* do */
+		if ((cur == NULL) || (cur == tree->parent))
+		    break; /* do */
 		if (cur->next != NULL) {
 		    cur = cur->next;
 		    if (xmlXIncludeTestNode(ctxt, cur))
@@ -2295,6 +2346,7 @@ xmlXIncludeProcessFlags(xmlDocPtr doc, int flags) {
     ctxt = xmlXIncludeNewContext(doc);
     if (ctxt == NULL)
 	return(-1);
+    ctxt->base = (xmlChar *)doc->URL;
     xmlXIncludeSetFlags(ctxt, flags);
     ret = xmlXIncludeDoProcess(ctxt, doc, tree);
     if ((ret >= 0) && (ctxt->nbErrors > 0))
@@ -2338,6 +2390,7 @@ xmlXIncludeProcessTreeFlags(xmlNodePtr tree, int flags) {
     ctxt = xmlXIncludeNewContext(tree->doc);
     if (ctxt == NULL)
 	return(-1);
+    ctxt->base = xmlNodeGetBase(tree->doc, tree);
     xmlXIncludeSetFlags(ctxt, flags);
     ret = xmlXIncludeDoProcess(ctxt, tree->doc, tree);
     if ((ret >= 0) && (ctxt->nbErrors > 0))
